<%@ page import="com.agillic.core.clm.otm.OneToManyPersonRecordsBean" %>
<%@ page import="java.text.SimpleDateFormat" %>
<%@ page import="java.util.*" %>
<%@ taglib prefix="sdk" uri="http://agillic.com/jsp/sdk/1.0" %>
<%@ taglib prefix="inp" uri="http://agillic.com/jsp/sdkinput/1.0" %>
<!-- In this example I do not use a receipt page but simply return to the same page for allowing multiple updates. This also means that the action is on the same page -->
<sdk:render>
    <inp:form next="index" method="post">
        <!-- Just a few basic fields to gather. Completely standard, since this is used on a logged in page the data will filled in -->
        <label>Email</label>
        <inp:text clmAttribute="EMAIL" class="required email unique textinput" transformer="trim,lower"/>

        <label>Firstname</label>
        <inp:text clmAttribute="FIRSTNAME" class="textinput"/>

        <label>Surname</label>
        <inp:text clmAttribute="LASTNAME" class="textinput"/>


        <!--
        So now we're going to build our selection of multiple colors
        You'll notice that I'm simply using standard html tags - using the Agillic tags will not really help us here
        To make sure we show the correct colors as selected we start by checking the otm data for which lines the user already has chosen
        -->
        <%
            //We start by getting the object holding the lines for the user
            OneToManyPersonRecordsBean colors = api.getPersonRecords(null, "COLORS");//we're logged in so null simply means "logged in user"
            //We need the index of the color and removed date columns in the returned data
            int colorIndex = colors.indexOfColumn("COLOR");
            int removedIndex = colors.indexOfColumn("REMOVED_DATE");
            Set<String> chosen = new HashSet<String>();
            //So just like a SQL ResultSet in java you can use next() to both check if there are more rows and to update the cursor to the newest
            while (colors.next()) {
                //We get the current row
                List<String> currentRow = colors.getCurrentRow();
                //And check the removed date - basically any value in removed date means the line is not active and should not be shown as selected
                String s = currentRow.get(removedIndex);
                if (s == null || s.equals("")) {
                    chosen.add(currentRow.get(colorIndex));
                }
            }
        %>
        <!-- Now we build the checkboxes and use the helper function to print if it should be selected or not -->
        <div>
            <input type="checkbox" name="color" value="red" <%=selected(chosen,"red")%>>Red
        </div>
        <div>
            <input type="checkbox" name="color" value="blue" <%=selected(chosen,"blue")%>>Blue
        </div>
        <div>
            <input type="checkbox" name="color" value="green" <%=selected(chosen,"green")%>>Green
        </div>
        <div>
            <input type="checkbox" name="color" value="yellow" <%=selected(chosen,"yellow")%>>Yellow
        </div>
        <div>
            <input type="checkbox" name="color" value="orange" <%=selected(chosen,"orange")%>>Orange
        </div>

        <input type="submit" value="Update">
    </inp:form>
</sdk:render>
<%!
    //Helper method for the render part
    public String selected(Set<String> sel, String color) {
        if (sel.contains(color)) {
            return "checked";
        } else {
            return "";
        }
    }
%>
<sdk:action>
    <%
        //First we start by updating the user - we get the fields from the form came from our tags and were tied to a Person Data
        Map<String, String> map = flow.getEnteredClmAttributes();
        //Simply call set clm attributes with the entered value and null for the user
        api.setClmAttributes(null,map);

        //In this example I have a One-To-Many table called COLORS - this has three fields COLOR (which is primary key), a date when the color was selected and a date when it was removed

        //First we start by finding out which lines the user already has - very much the same as above, however, we are both storing the current selected and removed lines in two different maps
        Map<String,String> alreadySelected = new HashMap<String,String>();
        Map<String,String> alreadyRemoved = new HashMap<String,String>();
        Set<String> selectedAfterUpdate = new HashSet<String>(); //this will come in handy later

        OneToManyPersonRecordsBean colorsData = api.getPersonRecords(null, "COLORS");
        int colorIndex = colorsData.indexOfColumn("COLOR");
        int removedIndex = colorsData.indexOfColumn("REMOVED_DATE");
        int curIndex = 0; //annoying but currently needed
        //Check the render phase comments to see details on the loop
        while (colorsData.next()) {
            List<String> currentRow = colorsData.getCurrentRow();
            String s = currentRow.get(removedIndex);
            //When you need to update data you should not provide the userid or the primary value of the row to the api but rather an internal rowid which uniquely defines the row - unfortunately you can currently only get that by given the rowIndex which is why we need our own counter
            //The loop below builds the mapping of chosen/removed colors to the relevant id for each
            if (s == null || s.equals("")) {
                alreadySelected.put(currentRow.get(colorIndex),colorsData.getRowId(curIndex));
            } else {
                alreadyRemoved.put(currentRow.get(colorIndex),colorsData.getRowId(curIndex));
            }
            curIndex++;
        }

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd.MM.yyyy"); //This is the default date format for api values - this should of course match what the system is configured to use
        String today = simpleDateFormat.format(new Date());
        //All the checkboxes had the same name which means that they will be represented as an array
        String[] colors = request.getParameterValues("color");
        List<Map<String, String>> oneToManyLines = new ArrayList<Map<String, String>>();
        //Null just to be sure to handle the case where the person didn't like any colors
        if (colors != null) {
            //We iterate over the colors and create each line
            for (String color : colors) {
                //We store all the ones selected - this will help us find lines that should be removed
                selectedAfterUpdate.add(color);
                if(!alreadySelected.containsKey(color)) {
                    //we're only interested in new selection
                    if(alreadyRemoved.containsKey(color)) {
                        //we have a line for this color - but it's removed, so we're going to update the line
                        //We mark that the line is active by removing the value in removed date, we also set a new selected date
                        Map<String,String> u = new HashMap<String,String>();
                        u.put("REMOVED_DATE","");
                        u.put("SELECTED_DATE",today);
                        api.updateOneToManyRecord(alreadyRemoved.get(color),"COLORS",u);
                    } else {
                        //Completely new lines let's add it
                        Map<String,String> u = new HashMap<String,String>();
                        u.put("COLOR",color);
                        u.put("SELECTED_DATE",today);
                        //new line so we use add instead of update
                        api.addOneToManyRecordForPerson(null,"COLORS",u);
                    }
                }
            }
        }
        //So now we know that selectedAfterUpdate contains the colors we should have
        //alreadySelected - contains the ones we had before we started the update we should now find any in that map that is no longer selected and set a removed date
        Set<Map.Entry<String, String>> entries = alreadySelected.entrySet();
        for (Map.Entry<String, String> entry : entries) {
            if (!selectedAfterUpdate.contains(entry.getKey())) {
                //should be removed
                Map<String, String> u = new HashMap<String, String>();
                u.put("REMOVED_DATE", today);
                api.updateOneToManyRecord(entry.getValue(), "COLORS", u); //remember the map contained color -> rowId
            }
        }
    %>
</sdk:action>